MAPPING = {
    "mirai2": "mirai",
    "BilibiliLive": "bilibili",
    "Console": "console",
    "Ding": "ding",
    "Discord": "discord",
    "Dodo": "dodo",
    "Feishu": "feishu",
    "Github": "github",
    "Kaiheila": "kook",
    "Minecraft": "minecraft",
    "ntchat": "ntchat",
    "OneBot V11": "onebot11",
    "OneBot V12": "onebot12",
    "QQ": "qq",
    "QQ Guild": "qqguild",
    "RedProtocol": "red",
    "Telegram": "telegram",
    "Satori": "satori",
    "Villa": "villa",
}
